@@ -23,18 +23,15 @@
 #include "mainwindow.h"
 
 
+bool Application::m_isKDE46OrHigher = false;
+
 Application::Application() : KUniqueApplication()
 {
     m_mainWindow = 0;
-}
 
-#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
-Application::Application(Display* display, Qt::HANDLE visual, Qt::HANDLE colormap)
-    : KUniqueApplication(display, visual, colormap)
-{
-    m_mainWindow = 0;
+    if (KDE::version() >= KDE_MAKE_VERSION(4, 6, 0))
+        m_isKDE46OrHigher = true;
 }
-#endif
 
 Application::~Application()
 {
