@@ -35,15 +35,16 @@ class Application : public KUniqueApplication
 
     public:
         explicit Application();
-#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
-        explicit Application(Display* display, Qt::HANDLE visual, Qt::HANDLE colormap);
-#endif
         virtual ~Application();
 
         virtual int newInstance();
 
+        static bool isKDE46OrHigher() { return m_isKDE46OrHigher; }
+
     private:
         MainWindow* m_mainWindow;
+
+        static bool m_isKDE46OrHigher;
 };
 
 #endif
