@@ -6,7 +6,7 @@
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor appro-
-  ved by the membership of KDE e.V.), which shall act as a proxy 
+  ved by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.
 
   This program is distributed in the hope that it will be useful,
@@ -20,15 +20,21 @@
 
 
 #include "windowsettings.h"
+#include "application.h"
 #include "settings.h"
 
 #include <QDesktopWidget>
 
+#include <kdeversion.h>
+
 
 WindowSettings::WindowSettings(QWidget* parent) : QWidget(parent)
 {
     setupUi(this);
 
+    if (!Application::isKDE46OrHigher())
+        kcfg_UseWMAssist->hide();
+
     for (int i = 2; i <= QApplication::desktop()->numScreens(); i++)
         kcfg_Screen->insertItem(i, i18nc("@item:inlistbox", "Screen <numid>%1</numid>", i));
 
