@@ -21,6 +21,7 @@
 
 
 #include "mainwindow.h"
+#include "application.h"
 #include "config/appearancesettings.h"
 #include "config/windowsettings.h"
 #include "firstrundialog.h"
@@ -51,13 +52,11 @@
 #include <QPainter>
 #include <QPalette>
 #include <QtDBus/QtDBus>
-#include <QTimer>
 
-#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
+#if defined(Q_WS_X11)
 #include <QX11Info>
 
 #include <X11/Xlib.h>
-#include <X11/extensions/Xrender.h>
 #endif
 
 
@@ -66,9 +65,7 @@ MainWindow::MainWindow(QWidget* parent)
 {
     QDBusConnection::sessionBus().registerObject("/yakuake/window", this, QDBusConnection::ExportScriptableSlots);
 
-#if KDE_IS_VERSION(4,2,68)
     setAttribute(Qt::WA_TranslucentBackground, true);
-#endif
 
     m_skin = new Skin();
     m_menu = new KMenu(this);
@@ -77,6 +74,11 @@ MainWindow::MainWindow(QWidget* parent)
     m_tabBar = new TabBar(this);
     m_titleBar = new TitleBar(this);
     m_firstRunDialog = NULL;
+    m_listenForActivationChanges = false;
+
+#if defined(Q_WS_X11)
+    m_kwinAssistPropSet = false;
+#endif
 
     setupActions();
     setupMenu();
@@ -98,6 +100,7 @@ MainWindow::MainWindow(QWidget* parent)
     connect(&m_mousePoller, SIGNAL(timeout()), this, SLOT(pollMouse()));
 
     connect(KWindowSystem::self(), SIGNAL(workAreaChanged()), this, SLOT(applyWindowGeometry()));
+    connect(KApplication::desktop(), SIGNAL(screenCountChanged(int)), this, SLOT(updateScreenMenu()));
 
     applySettings();
 
@@ -278,6 +281,34 @@ void MainWindow::setupActions()
     connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
     m_contextDependentActions << action;
 
+    action = actionCollection()->addAction("grow-terminal-right");
+    action->setText(i18nc("@action", "Grow Terminal to the Right"));
+    action->setIcon(KIcon("arrow-right"));
+    action->setShortcut(QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_Right));
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
+
+    action = actionCollection()->addAction("grow-terminal-left");
+    action->setText(i18nc("@action", "Grow Terminal to the Left"));
+    action->setIcon(KIcon("arrow-left"));
+    action->setShortcut(QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_Left));
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
+
+    action = actionCollection()->addAction("grow-terminal-top");
+    action->setText(i18nc("@action", "Grow Terminal to the Top"));
+    action->setIcon(KIcon("arrow-up"));
+    action->setShortcut(QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_Up));
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
+
+    action = actionCollection()->addAction("grow-terminal-bottom");
+    action->setText(i18nc("@action", "Grow Terminal to the Bottom"));
+    action->setIcon(KIcon("arrow-down"));
+    action->setShortcut(QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_Down));
+    connect(action, SIGNAL(triggered()), this, SLOT(handleContextDependentAction()));
+    m_contextDependentActions << action;
+
     action = actionCollection()->addAction("rename-session");
     action->setText(i18nc("@action", "Rename Session..."));
     action->setIcon(KIcon("edit-rename"));
@@ -372,6 +403,18 @@ void MainWindow::handleContextDependentAction(QAction* action, int sessionId)
 
     if (action == actionCollection()->action("split-top-bottom"))
         m_sessionStack->splitSessionTopBottom(sessionId);
+
+    if (action == actionCollection()->action("grow-terminal-right"))
+        m_sessionStack->tryGrowTerminalRight(m_sessionStack->activeTerminalId());
+
+    if (action == actionCollection()->action("grow-terminal-left"))
+        m_sessionStack->tryGrowTerminalLeft(m_sessionStack->activeTerminalId());
+
+    if (action == actionCollection()->action("grow-terminal-top"))
+        m_sessionStack->tryGrowTerminalTop(m_sessionStack->activeTerminalId());
+
+    if (action == actionCollection()->action("grow-terminal-bottom"))
+        m_sessionStack->tryGrowTerminalBottom(m_sessionStack->activeTerminalId());
 }
 
 void MainWindow::handleContextDependentToggleAction(bool checked, QAction* action, int sessionId)
@@ -540,7 +583,7 @@ void MainWindow::updateWindowHeightMenu()
 void MainWindow::configureKeys()
 {
     KShortcutsDialog::configure(actionCollection());
-    KWindowSystem::activateWindow(winId());
+    activate();
 }
 
 void MainWindow::configureApp()
@@ -667,11 +710,14 @@ void MainWindow::setWindowGeometry(int newWidth, int newHeight, int newPosition)
 
     int maxHeight = workArea.height() * newHeight / 100;
 
+    int targetWidth = workArea.width() * newWidth / 100;
+
     setGeometry(workArea.x() + workArea.width() * newPosition * (100 - newWidth) / 10000,
-                workArea.y(), workArea.width() * newWidth / 100, maxHeight);
+                workArea.y(), targetWidth, maxHeight);
 
     maxHeight -= m_titleBar->height();
-    m_titleBar->setGeometry(0, maxHeight, width(), m_titleBar->height());
+    m_titleBar->setGeometry(0, maxHeight, targetWidth, m_titleBar->height());
+    if (!isVisible()) m_titleBar->updateMask();
 
     if (Settings::frames() > 0)
         m_animationStepSize = maxHeight / Settings::frames();
@@ -765,21 +811,12 @@ void MainWindow::paintEvent(QPaintEvent* event)
 
     if (useTranslucency())
     {
-#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
-        painter.setCompositionMode(QPainter::CompositionMode_Source);
-        painter.fillRect(rect(), Qt::transparent);
-#endif
         painter.setOpacity(qreal(Settings::backgroundColorOpacity()) / 100);
         painter.fillRect(rect(), Settings::backgroundColor());
         painter.setOpacity(1.0);
     }
     else
-    {
-#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
-        painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
-#endif
         painter.fillRect(rect(), Settings::backgroundColor());
-    }
 
     QRect leftBorder(0, 0, m_skin->borderWidth(), height() - m_titleBar->height());
     painter.fillRect(leftBorder, m_skin->borderColor());
@@ -807,7 +844,7 @@ void MainWindow::moveEvent(QMoveEvent* event)
 
 void MainWindow::changeEvent(QEvent* event)
 {
-    if (event->type() == QEvent::ActivationChange)
+    if (m_listenForActivationChanges && event->type() == QEvent::ActivationChange)
     {
         if (isVisible() && !KApplication::activeWindow() && !Settings::keepOpen())
             toggleWindowState();
@@ -817,44 +854,145 @@ void MainWindow::changeEvent(QEvent* event)
 }
 
 void MainWindow::toggleWindowState()
+{
+    bool visible = isVisible();
+
+    if (visible && !isActiveWindow() && Settings::keepOpen() && Settings::toggleToFocus())
+    {
+        KWindowSystem::forceActiveWindow(winId());
+
+        return;
+    }
+
+#if defined(Q_WS_X11)
+    if (!Settings::useWMAssist() && m_kwinAssistPropSet)
+        kwinAssistPropCleanup();
+
+    if (Settings::useWMAssist() && KWindowSystem::compositingActive())
+        kwinAssistToggleWindowState(visible);
+    else
+#endif
+        xshapeToggleWindowState(visible);
+}
+
+#if defined(Q_WS_X11)
+void MainWindow::kwinAssistToggleWindowState(bool visible)
+{
+    // Always all back to legacy animation strategy if we're not on
+    // 4.6+. This block can be removed once we depend on KDE 4.6.
+    if (!Application::isKDE46OrHigher())
+    {
+        xshapeToggleWindowState(visible);
+
+        return;
+    }
+
+    bool gotEffect = false;
+
+    Display* display = QX11Info::display();
+    Atom atom = XInternAtom(display, "_KDE_SLIDE", false);
+    int count;
+    Atom* list = XListProperties(display, DefaultRootWindow(display), &count);
+
+    if (list != NULL)
+    {
+        gotEffect = (qFind(list, list + count, atom) != list + count);
+
+        XFree(list);
+    }
+
+    if (gotEffect)
+    {
+        Atom atom = XInternAtom(display, "_KDE_SLIDE", false);
+
+        if (Settings::frames() > 0)
+        {
+            QVarLengthArray<long, 1024> data(4);
+
+            data[0] = 0;
+            data[1] = 1;
+            data[2] = Settings::frames() * 10;
+            data[3] = Settings::frames() * 10;
+
+            XChangeProperty(display, winId(), atom, atom, 32, PropModeReplace,
+                reinterpret_cast<unsigned char *>(data.data()), data.size());
+
+            m_kwinAssistPropSet = true;
+        }
+        else
+            XDeleteProperty(display, winId(), atom);
+
+        if (visible)
+        {
+            sharedPreHideWindow();
+
+            hide();
+
+            sharedAfterHideWindow();
+        }
+        else
+        {
+            sharedPreOpenWindow();
+
+            show();
+
+            sharedAfterOpenWindow();
+        }
+
+        return;
+    }
+
+    // Fall back to legacy animation strategy if kwin doesn't have the
+    // effect loaded.
+    xshapeToggleWindowState(visible);
+}
+
+void MainWindow::kwinAssistPropCleanup()
+{
+    // We're not going to have set up the prop in the first place
+    // if we're not running in 4.6, so no need to clean it up.
+    // This block can be removed once we depend on KDE 4.6.
+    if (!Application::isKDE46OrHigher())
+        return;
+
+    Display* display = QX11Info::display();
+    Atom atom = XInternAtom(display, "_KDE_SLIDE", false);
+
+    XDeleteProperty(display, winId(), atom);
+
+    m_kwinAssistPropSet = false;
+}
+#endif
+
+void MainWindow::xshapeToggleWindowState(bool visible)
 {
     if (m_animationTimer.isActive()) return;
 
-    if (isVisible())
+    if (visible)
     {
-        if (!isActiveWindow() && Settings::keepOpen() && Settings::toggleToFocus())
-        {
-            KWindowSystem::forceActiveWindow(winId());
-            return;
-        }
+        sharedPreHideWindow();
 
         m_animationFrame = Settings::frames();
 
-        connect(&m_animationTimer, SIGNAL(timeout()), this, SLOT(retractWindow()));
+        connect(&m_animationTimer, SIGNAL(timeout()), this, SLOT(xshapeRetractWindow()));
         m_animationTimer.start();
     }
     else
     {
-        applyWindowGeometry();
-
         m_animationFrame = 0;
 
-        connect(&m_animationTimer, SIGNAL(timeout()), this, SLOT(openWindow()));
+        connect(&m_animationTimer, SIGNAL(timeout()), this, SLOT(xshapeOpenWindow()));
         m_animationTimer.start();
     }
 }
 
-void MainWindow::openWindow()
+void MainWindow::xshapeOpenWindow()
 {
     if (m_animationFrame == 0)
     {
-        updateUseTranslucency();
-
-        applyWindowProperties();
+        sharedPreOpenWindow();
 
         show();
-
-        if (Settings::pollMouse()) toggleMousePoll(false);
     }
 
     if (m_animationFrame == Settings::frames())
@@ -865,7 +1003,7 @@ void MainWindow::openWindow()
         m_titleBar->move(0, height() - m_titleBar->height());
         updateMask();
 
-        if (!Settings::firstRun()) KWindowSystem::forceActiveWindow(winId());
+        sharedAfterOpenWindow();
     }
     else
     {
@@ -883,7 +1021,7 @@ void MainWindow::openWindow()
     }
 }
 
-void MainWindow::retractWindow()
+void MainWindow::xshapeRetractWindow()
 {
     if (m_animationFrame == 0)
     {
@@ -892,7 +1030,7 @@ void MainWindow::retractWindow()
 
         hide();
 
-        if (Settings::pollMouse()) toggleMousePoll(true);
+        sharedAfterHideWindow();
     }
     else
     {
@@ -904,6 +1042,34 @@ void MainWindow::retractWindow()
     }
 }
 
+void MainWindow::sharedPreOpenWindow()
+{
+    applyWindowGeometry();
+
+    updateUseTranslucency();
+
+    applyWindowProperties();
+
+    if (Settings::pollMouse()) toggleMousePoll(false);
+}
+
+void MainWindow::sharedAfterOpenWindow()
+{
+    if (!Settings::firstRun()) KWindowSystem::forceActiveWindow(winId());
+
+    m_listenForActivationChanges = true;
+}
+
+void MainWindow::sharedPreHideWindow()
+{
+    m_listenForActivationChanges = false;
+}
+
+void MainWindow::sharedAfterHideWindow()
+{
+    if (Settings::pollMouse()) toggleMousePoll(true);
+}
+
 void MainWindow::activate()
 {
     KWindowSystem::activateWindow(winId());
@@ -931,7 +1097,12 @@ void MainWindow::pollMouse()
 
 void MainWindow::setKeepOpen(bool keepOpen)
 {
-    Settings::setKeepOpen(keepOpen);
+    if (Settings::keepOpen() != keepOpen)
+    {
+        Settings::setKeepOpen(keepOpen);
+
+        applyWindowProperties();
+    }
 
     actionCollection()->action("keep-open")->setChecked(keepOpen);
     m_titleBar->setFocusButtonState(keepOpen);
@@ -962,14 +1133,14 @@ int MainWindow::getScreen()
 QRect MainWindow::getDesktopGeometry()
 {
     QRect screenGeometry = KApplication::desktop()->screenGeometry(getScreen());
-    
+
     QAction* action = actionCollection()->action("view-full-screen");
 
     if (action->isChecked())
         return screenGeometry;
 
-    int currentDesktop = KWindowSystem::windowInfo(winId(), NET::CurrentDesktop).desktop();
-    
+    int currentDesktop = KWindowSystem::windowInfo(winId(), NET::WMDesktop).desktop();
+
     if (KApplication::desktop()->numScreens() > 1)
     {
         const QList<WId> allWindows = KWindowSystem::windows();
@@ -984,7 +1155,7 @@ QRect MainWindow::getDesktopGeometry()
             if (KWindowSystem::hasWId(windowId))
             {
                 KWindowInfo windowInfo = KWindowSystem::windowInfo(windowId, 0, NET::WM2ExtendedStrut);
-                
+
                 if (windowInfo.valid() && windowInfo.desktop() == currentDesktop)
                 {
                     NETExtendedStrut strut = windowInfo.extendedStrut();
@@ -1064,35 +1235,5 @@ void MainWindow::firstRunDialogOk()
 
 void MainWindow::updateUseTranslucency()
 {
-#if defined(Q_WS_X11) && !KDE_IS_VERSION(4,2,68)
-    bool ARGB = false;
-
-    int screen = QX11Info::appScreen();
-    bool depth = (QX11Info::appDepth() == 32);
-
-    Display* display = QX11Info::display();
-    Visual* visual = static_cast<Visual*>(QX11Info::appVisual(screen));
-
-    XRenderPictFormat* format = XRenderFindVisualFormat(display, visual);
-
-    if (depth && format->type == PictTypeDirect && format->direct.alphaMask)
-    {
-        ARGB = true;
-    }
-
-    if (ARGB && Settings::translucency())
-    {
-        m_useTranslucency = KWindowSystem::compositingActive();
-    }
-    else
-#elif KDE_IS_VERSION(4,2,68)
-    if (Settings::translucency())
-    {
-        m_useTranslucency = KWindowSystem::compositingActive();
-    }
-    else
-#endif
-    {
-        m_useTranslucency = false;
-    }
+    m_useTranslucency = (Settings::translucency() && KWindowSystem::compositingActive());
 }
