@@ -7,7 +7,7 @@
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor appro-
-  ved by the membership of KDE e.V.), which shall act as a proxy 
+  ved by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.
 
   This program is distributed in the hope that it will be useful,
@@ -64,7 +64,7 @@ void SessionStack::addSession(Session::SessionType type)
 
 void SessionStack::addSessionTwoHorizontal()
 {
-addSession(Session::TwoHorizontal);
+    addSession(Session::TwoHorizontal);
 }
 
 void SessionStack::addSessionTwoVertical()
@@ -243,7 +243,7 @@ void SessionStack::runCommandInTerminal(int terminalId, const QString& command)
 {
     QHashIterator<int, Session*> it(m_sessions);
 
-    while (it.hasNext()) 
+    while (it.hasNext())
     {
         it.next();
 
@@ -366,7 +366,6 @@ void SessionStack::splitTerminalLeftRight(int terminalId)
     int sessionId = sessionIdForTerminalId(terminalId);
 
     if (sessionId == -1) return;
-    if (!m_sessions.contains(sessionId)) return;
 
     m_sessions.value(sessionId)->splitLeftRight(terminalId);
 }
@@ -376,24 +375,59 @@ void SessionStack::splitTerminalTopBottom(int terminalId)
     int sessionId = sessionIdForTerminalId(terminalId);
 
     if (sessionId == -1) return;
-    if (!m_sessions.contains(sessionId)) return;
 
     m_sessions.value(sessionId)->splitTopBottom(terminalId);
 }
 
+int SessionStack::tryGrowTerminalRight(int terminalId, uint pixels)
+{
+    int sessionId = sessionIdForTerminalId(terminalId);
+
+    if (sessionId == -1) return -1;
+
+    return m_sessions.value(sessionId)->tryGrowTerminal(terminalId, Session::Right, pixels);
+}
+
+int SessionStack::tryGrowTerminalLeft(int terminalId, uint pixels)
+{
+    int sessionId = sessionIdForTerminalId(terminalId);
+
+    if (sessionId == -1) return -1;
+
+    return m_sessions.value(sessionId)->tryGrowTerminal(terminalId, Session::Left, pixels);
+}
+
+int SessionStack::tryGrowTerminalTop(int terminalId, uint pixels)
+{
+    int sessionId = sessionIdForTerminalId(terminalId);
+
+    if (sessionId == -1) return -1;
+
+    return m_sessions.value(sessionId)->tryGrowTerminal(terminalId, Session::Up, pixels);
+}
+
+int SessionStack::tryGrowTerminalBottom(int terminalId, uint pixels)
+{
+    int sessionId = sessionIdForTerminalId(terminalId);
+
+    if (sessionId == -1) return -1;
+
+    return m_sessions.value(sessionId)->tryGrowTerminal(terminalId, Session::Down, pixels);
+}
+
 void SessionStack::emitTitles()
 {
     QString title;
 
     QHashIterator<int, Session*> it(m_sessions);
 
-    while (it.hasNext()) 
+    while (it.hasNext())
     {
         it.next();
 
         title = it.value()->title();
 
-        if (!title.isEmpty()) 
+        if (!title.isEmpty())
             emit titleChanged(it.value()->id(), title);
     }
 }
