@@ -58,7 +58,9 @@ TabBar::TabBar(MainWindow* mainWindow) : QWidget(mainWindow)
     m_dropIndicator = 0;
 
     m_mainWindow = mainWindow;
+
     m_skin = mainWindow->skin();
+    connect(m_skin, SIGNAL(iconChanged()), this, SLOT(repaint()));
 
     m_tabContextMenu = new KMenu(this);
     connect(m_tabContextMenu, SIGNAL(hovered(QAction*)), this, SLOT(contextMenuActionHovered(QAction*)));
@@ -361,6 +363,28 @@ int TabBar::drawButton(int x, int y, int index, QPainter& painter)
     QFontMetrics fontMetrics(font);
     textWidth = fontMetrics.width(title) + 10;
 
+    // Draw the Prevent Closing image in the tab button.
+    if (m_mainWindow->sessionStack()->isSessionClosable(sessionId) == false)
+    {
+        if (selected)
+            painter.drawTiledPixmap(x, y,
+                    m_skin->tabBarPreventClosingImagePosition().x() +
+                    m_skin->tabBarPreventClosingImage().width(), height(), 
+                    m_skin->tabBarSelectedBackgroundImage());
+        else
+            painter.drawTiledPixmap(x, y, 
+                    m_skin->tabBarPreventClosingImagePosition().x() +
+                    m_skin->tabBarPreventClosingImage().width(), height(),
+                    m_skin->tabBarUnselectedBackgroundImage());
+
+        painter.drawPixmap(x + m_skin->tabBarPreventClosingImagePosition().x(),
+                           m_skin->tabBarPreventClosingImagePosition().y(),
+                           m_skin->tabBarPreventClosingImage());
+
+        x += m_skin->tabBarPreventClosingImagePosition().x();
+        x += m_skin->tabBarPreventClosingImage().width();
+    }
+
     if (selected)
         painter.drawTiledPixmap(x, y, textWidth, height(), m_skin->tabBarSelectedBackgroundImage());
     else
@@ -421,14 +445,15 @@ void TabBar::mousePressEvent(QMouseEvent* event)
 
     if (index == -1) return;
 
-    if (event->button() == Qt::LeftButton)
+    if (event->button() == Qt::LeftButton || event->button() == Qt::MidButton)
     {
         m_startPos = event->pos();
-        if (index != m_tabs.indexOf(m_selectedSessionId))
+        if (index != m_tabs.indexOf(m_selectedSessionId) || event->button() == Qt::MidButton)
         {
             m_mousePressed = true;
             m_mousePressedIndex = index;
         }
+        return;
     }
 
     QWidget::mousePressEvent(event);
@@ -442,20 +467,26 @@ void TabBar::mouseReleaseEvent(QMouseEvent* event)
 
     int index = tabAt(event->x());
 
-    if (event->button() == Qt::LeftButton && index != m_tabs.indexOf(m_selectedSessionId))
+    if (m_mousePressed && m_mousePressedIndex == index)
     {
-        if (m_mousePressed && m_mousePressedIndex == index)
+        if (event->button() == Qt::LeftButton && index != m_tabs.indexOf(m_selectedSessionId))
             emit tabSelected(m_tabs.at(index));
+
+        if (event->button() == Qt::MidButton)
+            emit tabClosed(m_tabs.at(index));
     }
 
     m_mousePressed = false;
 
+    m_startPos.setX(0);
+    m_startPos.setY(0);
+
     QWidget::mouseReleaseEvent(event);
 }
 
 void TabBar::mouseMoveEvent(QMouseEvent* event)
 {
-    if (!m_startPos.isNull() && (event->buttons() & Qt::LeftButton))
+    if (!m_startPos.isNull() && ((event->buttons() & Qt::LeftButton) || (event->buttons() & Qt::MidButton)))
     {
         int distance = (event->pos() - m_startPos).manhattanLength();
 
@@ -544,7 +575,7 @@ void TabBar::dropEvent(QDropEvent* event)
             --targetIndex;
 
         m_tabs.move(sourceIndex, targetIndex);
-        selectTab(sourceSessionId);
+        emit tabSelected(m_tabs.at(targetIndex));
 
         event->accept();
     }
@@ -619,13 +650,13 @@ void TabBar::interactiveRename(int sessionId)
     if (sessionId == -1) return;
     if (!m_tabs.contains(sessionId)) return;
 
+    m_renamingSessionId = sessionId;
+
     int index = m_tabs.indexOf(sessionId);
     int x = index ? m_tabWidths.at(index - 1) : m_skin->tabBarPosition().x();
     int y = m_skin->tabBarPosition().y();
     int width = m_tabWidths.at(index) - x;
 
-    m_renamingSessionId = index;
-
     m_lineEdit->setText(m_tabTitles[sessionId]);
     m_lineEdit->setGeometry(x-1, y-1, width+3, height()+2);
     m_lineEdit->selectAll();
@@ -773,7 +804,7 @@ QString TabBar::standardTabTitle()
         if (!nameOk)
         {
             count++;
-            newTitle = newTitle = makeTabTitle(count);
+            newTitle = makeTabTitle(count);
         }
     }
     while (!nameOk);
